/**
 *
 */
package com.diagbot.facade;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.GetRetrievalsByTagDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalListDTO;
import com.diagbot.entity.Retrieval;
import com.diagbot.entity.RetrievalMapping;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.RetrievalServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddTagRetrievalDetailVO;
import com.diagbot.vo.AddTagRetrievalVO;
import com.diagbot.vo.DelRetrievalsByMapsVO;
import com.diagbot.vo.GetRetrievalsByTagVO;
import com.diagbot.vo.RetrievalListVO;

/**
 * @author rgb
 * @Description
 * @time 2018年12月5日下午4:53:59
 */
@Component
public class RetrievalFacade extends RetrievalServiceImpl {

    @Autowired
    private RetrievalMappingFacade retrievalMappingFacade;
    @Autowired
    private UserServiceClient userServiceClient;

    /**
     * 根据标签获取同义词
     *
     * @param getRetrievalsByTagVO
     * @return
     */
    public List<GetRetrievalsByTagDTO> getRetrievalsByTag(GetRetrievalsByTagVO getRetrievalsByTagVO) {

        return this.baseMapper.getRetrievalsByTag(getRetrievalsByTagVO);
    }

    /**
     * 根据映射关系删除标签的同义词
     *
     * @param delRetrievalsByMapsVO
     * @return
     */
    public Boolean delRetrievalsByMaps(DelRetrievalsByMapsVO delRetrievalsByMapsVO) {
    	QueryWrapper<RetrievalMapping> mappingQe = new QueryWrapper<>();
		String userId = UserUtils.getCurrentPrincipleID();
		RetrievalMapping retrievalMapping = new RetrievalMapping();
		retrievalMapping.setGmtModified(DateUtil.now());
		retrievalMapping.setModifier(userId);
		retrievalMapping.setIsDeleted("Y");
		mappingQe.in("id", delRetrievalsByMapsVO.getRetrievalMappingIds());
		retrievalMappingFacade.update(retrievalMapping, mappingQe);
		return true;
    }

    /**
     * 标签添加同义词
     *
     * @param addTagRetrievalVO
     * @return
     */
    public Boolean addTagRetrieval(AddTagRetrievalVO addTagRetrievalVO) {
    	Date now = DateUtil.now();
    	String userId = UserUtils.getCurrentPrincipleID();
    	
        Map<String,AddTagRetrievalDetailVO> detailMap = addTagRetrievalVO.getItemList().stream().collect(Collectors.toMap(AddTagRetrievalDetailVO::getRetrievalName,i->i));
    	List<String> retrievalNames = addTagRetrievalVO.getItemList().stream().map(i->i.getRetrievalName()).collect(Collectors.toList());
    	
        QueryWrapper<Retrieval> retrievalQe = new QueryWrapper<Retrieval>();
        retrievalQe.in("name", retrievalNames);
        retrievalQe.eq("is_deleted", "N");
        Map<String,Retrieval> retrievalMap = this.list(retrievalQe).stream().collect(Collectors.toMap(Retrieval::getName,i->i));
        
        List<Retrieval> saveOrUpdateRetrievals = new ArrayList<>();
        List<Retrieval> retrievals = new ArrayList<>();
        addTagRetrievalVO.getItemList().forEach(i->{
        	Retrieval retrieval = retrievalMap.get(i.getRetrievalName());
        	if(retrieval==null){
        		retrieval = new Retrieval();
        		retrieval.setName(i.getRetrievalName());
        		retrieval.setSpell(i.getRetrievalSpell());
        		retrieval.setCreator(userId);
        		retrieval.setGmtCreate(now);
        		retrieval.setModifier(userId);
        		retrieval.setGmtModified(now);
        		saveOrUpdateRetrievals.add(retrieval);
        	}else if(!retrieval.getSpell().equals(i.getRetrievalSpell())){
        		retrieval.setSpell(i.getRetrievalSpell());
        		retrieval.setModifier(userId);
        		retrieval.setGmtModified(now);
        		saveOrUpdateRetrievals.add(retrieval);
        	}else{
        		retrievals.add(retrieval);
        	}
        });
        this.saveOrUpdateBatch(saveOrUpdateRetrievals);
        retrievals.addAll(saveOrUpdateRetrievals);
        List<Long> retrievalIds = retrievals.stream().map(i->i.getId()).collect(Collectors.toList());
        
    	QueryWrapper<RetrievalMapping> retrievalMappingQe = new QueryWrapper<RetrievalMapping>();
    	retrievalMappingQe.eq("question_id", addTagRetrievalVO.getQuestionId());
        List<RetrievalMapping> retrievalMappings = retrievalMappingFacade.list(retrievalMappingQe);
        Map<Long,RetrievalMapping> retrievalMappingMap = retrievalMappings.stream().collect(Collectors.toMap(RetrievalMapping::getRetrievalId, i->i));
        
        List<RetrievalMapping> addOrUpdateRetrievalMappings = new ArrayList<>();
        retrievalMappings.stream().filter(i->retrievalIds.contains(i.getRetrievalId())).forEach(i->{
        	i.setIsDeleted("Y");
        	i.setGmtModified(now);
        	i.setModifier(userId);
        	addOrUpdateRetrievalMappings.add(i);
        });
        
        retrievals.forEach(i->{
        	RetrievalMapping retrievalMapping = retrievalMappingMap.get(i.getId());
        	if(retrievalMapping==null){
        		retrievalMapping = new RetrievalMapping();
        		retrievalMapping.setQuestionId(addTagRetrievalVO.getQuestionId());
        		retrievalMapping.setRetrievalId(i.getId());
        		retrievalMapping.setGmtCreate(now);
        		retrievalMapping.setGmtModified(now);
        		retrievalMapping.setCreator(userId);
        		retrievalMapping.setModifier(userId);
        	}else{
        		if(detailMap.get(i.getName()).getRetrievalType()!=retrievalMapping.getShowType()){
        			retrievalMapping.setShowType(detailMap.get(i.getName()).getRetrievalType());
        			retrievalMapping.setGmtModified(now);
        			retrievalMapping.setModifier(userId);
        		}
        	}
        	addOrUpdateRetrievalMappings.add(retrievalMapping);
        });
        
        retrievalMappingFacade.saveOrUpdateBatch(addOrUpdateRetrievalMappings);     

        return true;
    }
    
    /**
	 * 获取标签同义词列表
	 * @param retrievalListVO
	 * @return
	 */
    public IPage<RetrievalListDTO> retrievalList(RetrievalListVO retrievalListVO) {
        IPage<RetrievalListDTO> ipage = this.getRetrievalList(retrievalListVO);
        List<String> ids = ipage.getRecords().stream().map(i->i.getOperatorName()).collect(Collectors.toList());
        RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(ids);
        if (respDTO == null || !"0".equals(respDTO.code)) {
            throw new CommonException(CommonErrorCode.RPC_ERROR,
                    "获取用户信息失败");
        }
        ipage.getRecords().forEach(i->i.setOperatorName(respDTO.data.get(i.getOperatorName())));
        return ipage;
    }

}
